from setuptools import setup

setup(name='brotz',
      version='0.1.0',
      description='Lightweight, chiller way to compose html in python',
      url='http://github.com/leontrolski/brotz',
      author='leontrolski',
      author_email='ojhrussell@gmail.com',
      license='GNU General Public License v3.0',
      packages=['brotz'],
      zip_safe=False)
